import os
from datetime import datetime, timedelta

from airflow import DAG
from airflow.contrib.operators.kubernetes_pod_operator import KubernetesPodOperator
from airflow_utils import (
    DATA_IMAGE,
    clone_and_setup_extraction_cmd,
    gitlab_defaults,
    slack_failed_task,
)

from kube_secrets import (
    QUALTRICS_API_TOKEN,
    QUALTRICS_POOL_ID,
    QUALTRICS_NPS_ID,
    QUALTRICS_POST_PURCHASE_ID,
    SNOWFLAKE_ACCOUNT,
    SNOWFLAKE_LOAD_DATABASE,
    SNOWFLAKE_LOAD_PASSWORD,
    SNOWFLAKE_LOAD_ROLE,
    SNOWFLAKE_LOAD_USER,
    SNOWFLAKE_LOAD_WAREHOUSE,
)
from kubernetes_helpers import get_affinity, get_toleration


env = os.environ.copy()
pod_env_vars = {"CI_PROJECT_DIR": "/analytics"}

default_args = {
    "depends_on_past": False,
    "on_failure_callback": slack_failed_task,
    "owner": "airflow",
    "retries": 1,
    "retry_delay": timedelta(minutes=1),
    "sla": timedelta(hours=12),
    "sla_miss_callback": slack_failed_task,
    "start_date": datetime(2023, 8, 24),
    "dagrun_timeout": timedelta(hours=6),
}

dag = DAG(
    "qualtrics_survey_extract",
    default_args=default_args,
    schedule_interval="0 */12 * * *",
    catchup=True,
)

# don't add a newline at the end of this because it gets added to in the K8sPodOperator arguments
qualtrics_extract_command = f"{clone_and_setup_extraction_cmd} && python qualtrics/src/download_survey_responses.py"

qualtrics_operator = KubernetesPodOperator(
    **gitlab_defaults,
    image=DATA_IMAGE,
    task_id="qualtrics-survey-extract",
    name="qualtrics-survey-extract",
    secrets=[
        QUALTRICS_API_TOKEN,
        QUALTRICS_NPS_ID,
        QUALTRICS_POOL_ID,
        QUALTRICS_POST_PURCHASE_ID,
        SNOWFLAKE_ACCOUNT,
        SNOWFLAKE_LOAD_DATABASE,
        SNOWFLAKE_LOAD_ROLE,
        SNOWFLAKE_LOAD_USER,
        SNOWFLAKE_LOAD_WAREHOUSE,
        SNOWFLAKE_LOAD_PASSWORD,
    ],
    env_vars={
        **pod_env_vars,
        **{
            "QUALTRICS_DATA_CENTER": "eu",
        },
    },
    affinity=get_affinity("extraction"),
    tolerations=get_toleration("extraction"),
    arguments=[qualtrics_extract_command],
    dag=dag,
)
