""" This module is for sheetload/driveload/s3 processes"""

import sys
import re
from io import StringIO
import json
from logging import error, info, basicConfig, getLogger, warning
from os import environ as env
from typing import Dict, Tuple, List
from yaml import load, safe_load, YAMLError, FullLoader

import boto3
import pandas as pd
from fire import Fire
from gitlabdata.orchestration_utils import (
    snowflake_engine_factory,
    query_executor,
)
from google_sheets_client import GoogleSheetsClient
from google_drive_client import GoogleDriveClient
from google.cloud import storage
from google.oauth2 import service_account
from sheetload_dataframe_utils import dw_uploader
from sheetload_dataframe_utils import dw_uploader_append_only
from qualtrics_sheetload import qualtrics_loader

from pandas.errors import ParserError


def sheet_loader(
    sheet_file: str,
    table_name: str = None,
    schema: str = "sheetload",
    database: str = "RAW",
    gapi_keyfile: str = None,
    conn_dict: Dict[str, str] = None,
) -> None:
    """
    Load data from a google sheet into a DataFrame and pass it to dw_uploader.
    The sheet must have been shared with the google service account of the runner.

    Loader expects the name of the sheet to be:
    <sheet_name>.<tab>
    The tab name will become the table name.

    Column names can not contain parentheses. Spaces and slashes will be
    replaced with underscores.

    sheet_file: path to yaml file with sheet configurations

    table_name: Optional, name of the tab to be loaded -- matches the table_name as well as part of the document name
      -- For example for the test sheet this should be "test_sheet" as that is the name of the tab to be loaded from the document.
      -- Also should match the second half of the sheet name -- for example `sheetload.test_sheet`.
      -- Also the name of the final table in Snowflake.  The test sheet turns into a RAW.SHEETLOAD.test_sheet table.

    python sheetload.py sheets <sheet_file>
    """

    with open(sheet_file, "r") as file:
        try:
            stream = safe_load(file)
        except YAMLError as exc:
            print(exc)

        sheets = [
            (sheet["name"], tab)
            for sheet in stream["sheets"]
            for tab in sheet["tabs"]
            if (table_name is None or tab == table_name)
        ]

    if database != "RAW":
        engine = snowflake_engine_factory(conn_dict or env, "ANALYTICS_LOADER", schema)
        database = env["SNOWFLAKE_PROD_DATABASE"]
        # Trys to create the schema its about to write to
        # If it does exists, {schema} already exists, statement succeeded.
        # is returned.
        schema_check = f"""CREATE SCHEMA IF NOT EXISTS "{database}".{schema}"""
        query_executor(engine, schema_check)
    else:
        engine = snowflake_engine_factory(conn_dict or env, "LOADER", schema)

    info(engine)

    # Get the credentials for sheets and the database engine

    google_sheet_client = GoogleSheetsClient()

    for sheet_name, tab in sheets:
        info(f"Processing sheet: {sheet_name}")

        dataframe = google_sheet_client.load_google_sheet(
            gapi_keyfile, schema + "." + sheet_name, tab
        )
        dw_uploader(engine, tab, dataframe, schema)
        info(f"Finished processing for table: {tab}")

    query = f"""grant select on all tables in schema "{database}".{schema} to role transformer"""
    query_executor(engine, query)
    info("Permissions granted.")


def gcs_loader(
    path: str,
    bucket: str,
    schema: str = "sheetload",
    compression: str = "gzip",
    conn_dict: Dict[str, str] = None,
    gapi_keyfile: str = None,
) -> None:
    """
    Download a CSV file from a GCS bucket and then pass it to dw_uploader.

    Loader expects <table_name>.*

    Column names can not contain parentheses. Spaces and slashes will be
    replaced with underscores.

    Paths is a list that is separated spaces. i.e.:
    python sheetload.py gcs --bucket <bucket> --destination <snowflake|postgres>  <path_1> <path_2> ...
    """

    # Set some vars
    chunksize = 15000
    chunk_iter = 0

    engine = snowflake_engine_factory(conn_dict or env, "LOADER", schema)

    # Get the gcloud storage client and authenticate
    scope = ["https://www.googleapis.com/auth/cloud-platform"]
    keyfile = load(gapi_keyfile or env["GCP_SERVICE_CREDS"], Loader=FullLoader)
    credentials = service_account.Credentials.from_service_account_info(keyfile)
    scoped_credentials = credentials.with_scopes(scope)
    storage_client = storage.Client(credentials=scoped_credentials)
    bucket_obj = storage_client.get_bucket(bucket)

    # Download the file and then pass it in chunks to dw_uploader
    blob = bucket_obj.blob(path)
    blob.download_to_filename(path)
    table = path.split(".")[0]

    try:
        sheet_df = pd.read_csv(
            path,
            engine="c",
            low_memory=False,
            compression=compression,
            chunksize=chunksize,
        )
    except FileNotFoundError:
        info("File {} not found.".format(path))

    # Upload each chunk of the file
    for chunk in sheet_df:
        chunk[chunk.columns] = chunk[chunk.columns].astype("str")
        dw_uploader(engine=engine, table=table, data=chunk, chunk=chunk_iter)
        chunk_iter += 1
    max_size_of_relation = chunk_iter * chunksize
    min_size_of_relation = max((chunk_iter - 1) * chunksize, 0)
    actual_size = query_executor(f"SELECT COUNT(*) FROM {table};", engine)[0][0]
    if (actual_size > max_size_of_relation) or (actual_size < min_size_of_relation):
        error(
            f"Count in Snowflake for table {table} ({actual_size})"
            " did not match the range of what was read in code "
            f"({min_size_of_relation} to {max_size_of_relation})"
        )
        sys.exit(1)


def count_records_in_s3_csv(bucket: str, s3_file_key: str, s3_client) -> int:
    """
    This function is used to count the number of records found in a CSV on S3 with path
    s3://bucket/s3_file_key .  The number of records is returned.

    This function uses an AWS feature known as "s3 select" which can perform queries
    directly on s3 objects.  This function assumes that the CSVs it is querying have header rows at the top.
    """

    query_result = s3_client.select_object_content(
        Bucket=bucket,
        Key=s3_file_key,
        ExpressionType="SQL",
        Expression="select count(*) as line_count from s3object",
        InputSerialization={
            "CSV": {"FileHeaderInfo": "IGNORE", "AllowQuotedRecordDelimiter": True}
        },
        OutputSerialization={"JSON": {}},
    )
    for event in query_result["Payload"]:
        if "Records" in event:
            json_payload_dict = json.loads(event["Records"]["Payload"].decode("utf-8"))
            return json_payload_dict["line_count"]
    return -1


def check_s3_csv_count_integrity(
    bucket, file_key, s3_client, snowflake_engine, table_name
) -> None:
    """
    This function is used to verify that the count of rows in the snowflake table with name "table_name"
    is equivalent to the count of records found in the csv on aws s3 with path s3://bucket/file_key .

    If the counts are equal, this function returns gracefully and returns nothing.  If the counts are not equal,
    this function logs an error and exits the current running program with an exit code of 1.  The exit code is
    used to signal airflow that it should fail the task.
    """
    snowflake_count_result_set = query_executor(
        snowflake_engine, f"select count(*) from {table_name}"
    )
    snowflake_count = snowflake_count_result_set[0][0]
    s3_count = count_records_in_s3_csv(bucket, file_key, s3_client)
    if snowflake_count != s3_count:
        error(
            f"Error replicating CSV from S3 for table name: {table_name}. Snowflake count: {snowflake_count}, S3 count: {s3_count}."
        )
        sys.exit(1)


def get_s3_credentials(schema: str) -> tuple:
    """
    This function returns the set of aws_access_key_id,aws_secret_access_key,path_prefix
    based on the the schema name provided.
    """

    if schema == "greenhouse":
        aws_access_key_id = env["GREENHOUSE_ACCESS_KEY_ID"]
        aws_secret_access_key = env["GREENHOUSE_SECRET_ACCESS_KEY"]
        path_prefix = ""
    elif schema == "gainsight":
        aws_access_key_id = env["GAINSIGHT_ACCESS_KEY_ID"]
        aws_secret_access_key = env["GAINSIGHT_SECRET_ACCESS_KEY"]
        # The file is located in  subfolder fixed by gainsight team.
        path_prefix = "gainsight-to-snowflake/"
    return aws_access_key_id, aws_secret_access_key, path_prefix


def s3_loader(bucket: str, schema: str, conn_dict: Dict[str, str] = None) -> None:
    """
    Load data from csv files stored in an S3 Bucket into a DataFrame and pass it to dw_uploader
    for loading into Snowflake.

    Loader will iterate through all files in the provided bucket that have the `.csv` extension.

    python sheetload.py s3 --bucket datateam-greenhouse-extract --schema greenhouse

    """

    # Create Snowflake engine
    engine = snowflake_engine_factory(conn_dict or env, "LOADER", schema)

    # Set S3 Client variable
    aws_access_key_id, aws_secret_access_key, path_prefix = get_s3_credentials(schema)

    session = boto3.Session(
        aws_access_key_id=aws_access_key_id, aws_secret_access_key=aws_secret_access_key
    )
    s3_client = session.client("s3")
    s3_bucket = s3_client.list_objects(Bucket=bucket, Prefix=path_prefix)

    # Iterate through files and upload
    for obj in s3_bucket["Contents"]:
        file = obj["Key"]

        if (
            file != "application_question_answers.csv"
        ):  # Temporary hotfix to address https://gitlab.com/gitlab-data/analytics/-/issues/19972#note_1806135779

            info(f"Working on {file}...")

            if re.search(r"\.csv", file):
                csv_obj = s3_client.get_object(Bucket=bucket, Key=file)
                body = csv_obj["Body"]
                csv_string = body.read().decode("utf-8")
                try:
                    sheet_df = pd.read_csv(
                        StringIO(csv_string), engine="c", low_memory=False
                    )

                    table_name, extension = file.split(".")[0:2]
                    # To pick up the file name alone  not the whole path to the file for as table name
                    table = table_name.split("/")[-1]

                    dw_uploader(engine, table, sheet_df, truncate=True)

                    check_s3_csv_count_integrity(bucket, file, s3_client, engine, table)

                except ParserError:
                    error(f"Problem processing {file}")


def csv_loader(
    filename: str,
    schema: str,
    database: str = "RAW",
    tablename: str = None,
    header: str = "infer",
    conn_dict: Dict[str, str] = None,
):
    """
    Loads csv files from a local file system into a DataFrame and pass it to dw_uploader
    for loading into Snowflake.

    Tablename will use the name of the csv by default.
    python sheetload.py csv --filename nvd.csv --schema engineering_extracts
    becomes raw.engineering_extracts.nvd

    Header will read the first row of the csv as the column names by default. Passing
    None will use integers for each column.

    python sheetload.py csv --filename nvd.csv --schema engineering_extracts --tablename nvd_data --header None
    """

    # Create Snowflake engine
    engine = snowflake_engine_factory(conn_dict or env, "LOADER", schema)
    info(engine)

    csv_data = pd.read_csv(filename, header=header)

    if tablename:
        table = tablename
    else:
        table = filename.split(".")[0].split("/")[-1]

    info(f"Uploading {filename} to {database}.{schema}.{table}")

    dw_uploader(engine, table=table, data=csv_data, schema=schema, truncate=True)


def drive_loader(
    drive_file: str,
    schema: str = "driveload",
    table_name: str = None,
    gapi_keyfile: str = None,
    conn_dict: Dict[str, str] = None,
):
    """
    Load all data from a Google Drive Folder into a DataFrame and pass it to dw_uploader.
    The folder must have been shared with the google service account of the runner.

    All files from one folder are uploaded into one table.

    This process creates an "archived" folder and moves files into this folder once uploaded

    Column names can not contain parentheses. Spaces and slashes will be
    replaced with underscores.

    drive: path to yaml file with folder configurations

    table_name: Optional, matches the table_name to be loaded

    For a working example of this function being called see the driveload DAG.

    python driveload.py drive <drive_file>
    """
    engine = snowflake_engine_factory(conn_dict or env, "LOADER", schema)
    info(engine)

    google_drive_client = GoogleDriveClient(gapi_keyfile)

    with open(drive_file, "r") as yaml_file:
        stream = safe_load(yaml_file)

        folders = [
            folder
            for folder in stream["folders"]
            if (table_name is None or folder.get("table_name") == table_name)
        ]

    for folder in folders:
        folder_name = folder.get("folder_name")
        table_name = folder.get("table_name")
        # If chunk is NULL or not passed then the current code behavior is to do the replace.
        chunk = folder.get("table_replace_append")
        # mypy fix
        if table_name is None:
            raise ValueError

        info(f"Processing folder {folder_name}")

        folder_id = google_drive_client.get_item_id(
            item_name=folder_name, is_folder=True
        )
        archive_folder_id = google_drive_client.get_archive_folder_id(
            in_folder_id=folder_id
        )

        files = google_drive_client.get_files_in_folder(
            folder_id=folder_id, file_type="text/csv"
        )
        info(f"Found {str(len(files))} to process")

        for file in files:
            file_id = file.get("id", "")
            data = google_drive_client.get_data_frame_from_file_id(file_id=file_id)
            dw_uploader_append_only(engine, table=table_name, data=data, chunk=chunk)

            if archive_folder_id:
                info("Moving file to archive folder")
                google_drive_client.move_file_to_folder(
                    file_id=file_id, to_folder_id=archive_folder_id
                )


if __name__ == "__main__":
    basicConfig(stream=sys.stdout, level=20)
    getLogger("snowflake.connector.cursor").disabled = True
    Fire(
        {
            "sheets": sheet_loader,
            "gcs": gcs_loader,
            "s3": s3_loader,
            "csv": csv_loader,
            "qualtrics": qualtrics_loader,
            "drive": drive_loader,
        }
    )
    info("Complete.")
